/** 
 * Copyright (c) 2010 Keang Ltd
 * 
 * This file is part of the Hexagonal Buttons package.
 *
 * The Hexagonal Buttons package is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published 
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The Hexagonal Buttons package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with the Hexagonal Buttons package.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.co.keang.hex.demo;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.EmptyBorder;
import uk.co.keang.hex.gui.HexButton;
import uk.co.keang.hex.gui.HexLayout;
import uk.co.keang.hex.gui.HexToggleButton;
import uk.co.keang.hex.gui.HexBtnUtilities.HexOrientation;
import uk.co.keang.hex.gui.HexLayout.HexLayoutStyle;

/**
 * A demonstration program for Hexagonal buttons and layout
 *
 * @author A.G.Docherty
 * @date 19 Jun 2009
 * @version 1.0
 *
 */
public class HexLayoutDemo extends JPanel
{
private static final long serialVersionUID = 5759426245186257956L;
private static final String BUTTON = "Momentary";
private static final String TOGGLEBUTTON = "Toggle";
private JPanel hexPanel;
private boolean stretchy;
private int nLines;
private int nCpl;
private int cGap;
private int nComponents;
private HexOrientation orientation;
private HexLayoutStyle style;
protected String btnType;

HexLayoutDemo()
    {
    nLines = 4;
    nCpl = 4;
    cGap = 1;
    nComponents = 12;
    orientation = HexOrientation.VERTICAL_SIDES;
    style = HexLayoutStyle.STRAIGHT_MAX_FIRST;
    btnType = TOGGLEBUTTON;

    init();
    }

private void init()
    {
    setLayout(new BorderLayout());

    JPanel p = new JPanel();
    p.setBorder(new EmptyBorder(10, 5, 10, 5));
    p.setLayout(new GridLayout(0, 6, 5, 10));

    p.add(new JLabel("Num Lines", SwingConstants.RIGHT));
    JComboBox cb = new JComboBox(new Integer[]{0,1,2,3,4,5,6,7,8,9,10});
    cb.setSelectedItem(nLines);
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            nLines = (Integer)box.getSelectedItem();
            updateLayout();
            }});
    p.add(cb);

    p.add(new JLabel("Items per Line", SwingConstants.RIGHT));
    cb = new JComboBox(new Integer[]{0,1,2,3,4,5,6,7,8,9,10});
    cb.setSelectedItem(nCpl);
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            nCpl = (Integer)box.getSelectedItem();
            updateLayout();
            }});
    p.add(cb);

    p.add(new JLabel("Gap", SwingConstants.RIGHT));
    cb = new JComboBox(new Integer[]{0,1,2,3,4,5,6,7,8,9,10});
    cb.setSelectedItem(cGap);
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            cGap = (Integer)box.getSelectedItem();
            updateLayout();
            }});
    p.add(cb);

    p.add(new JLabel("Num Components", SwingConstants.RIGHT));
    cb = new JComboBox(new Integer[]{0,1,2,3,4,5,6,7,8,9,10,12,13,14,15,16,17,18,19,20,25,30,40});
    cb.setSelectedItem(nComponents);
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            nComponents = (Integer)box.getSelectedItem();
            hexPanel.removeAll();
            hexPanel.revalidate();
            repaint();
            createComponents();
            updateLayout();
            }});
    p.add(cb);

    p.add(new JLabel("Hex Shape", SwingConstants.RIGHT));
    cb = new JComboBox(new String[]{"Proper Hexagon", "Stretch to fill space"});
    cb.setSelectedIndex(stretchy?1:0);
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            int index = box.getSelectedIndex();

            stretchy = index == 0?false:true;
            updateLayout();
            }});
    p.add(cb);

    p.add(new JLabel("Orientation", SwingConstants.RIGHT));
    cb = new JComboBox(new String[]{HexOrientation.VERTICAL_SIDES.name(), HexOrientation.HORIZONTAL_SIDES.name()});
    cb.setSelectedItem(orientation.name());
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            orientation = HexOrientation.valueOf((String)box.getSelectedItem());
            updateLayout();
            }});
    p.add(cb);

    p.add(new JLabel("Layout Style", SwingConstants.RIGHT));
    cb = new JComboBox(new String[]{HexLayoutStyle.WAVEY.name(), HexLayoutStyle.STRAIGHT_MIN_FIRST.name(), HexLayoutStyle.STRAIGHT_MAX_FIRST.name()});
    cb.setSelectedItem(style.name());
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            style = HexLayoutStyle.valueOf((String)box.getSelectedItem());
            updateLayout();
            }});
    p.add(cb);

    p.add(new JLabel("Button Type", SwingConstants.RIGHT));
    cb = new JComboBox(new String[]{BUTTON, TOGGLEBUTTON});
    cb.setSelectedItem(btnType);
    cb.addActionListener(new ActionListener(){
        @Override
        public void actionPerformed(ActionEvent event)
            {
            JComboBox box = (JComboBox)event.getSource();
            btnType = (String)box.getSelectedItem();
            hexPanel.removeAll();
            hexPanel.revalidate();
            repaint();
            createComponents();
            updateLayout();
            }});
    p.add(cb);

    add(p, BorderLayout.SOUTH);

    hexPanel = new JPanel();
    setHexPanelLayout();
    createComponents();

    add(hexPanel, BorderLayout.CENTER);
    }

private void setHexPanelLayout()
    {
    HexLayout hexLayout = new HexLayout(nCpl, nLines, cGap, orientation, style);
    hexLayout.setStretchy(stretchy);
    hexPanel.setLayout(hexLayout);
    }

private void createComponents()
    {
    Icon icon = null;

    URL url = HexLayoutDemo.class.getResource("pressIcon.gif");
    if ( url != null )
        icon = new ImageIcon(url);

    for ( int i = 0; i < nComponents; i++ )
        {
        AbstractButton b2 = null;

        if ( btnType.equals(TOGGLEBUTTON) )
            b2 = new HexToggleButton("B"+(i+1), icon);
        else
            b2 = new HexButton("B"+(i+1), icon);

        hexPanel.add(b2);
        }

//    hexPanel.add(new JButton("t", icon));
    }

private void updateLayout()
    {
    setHexPanelLayout();
    this.revalidate();
    }

/**
 * Application Entry Point
 */
public static void main(String[] args)
    {
    try
        {
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
    catch ( ClassNotFoundException e )
        {
        e.printStackTrace();
        }
    catch ( InstantiationException e )
        {
        e.printStackTrace();
        }
    catch ( IllegalAccessException e )
        {
        e.printStackTrace();
        }
    catch ( UnsupportedLookAndFeelException e )
        {
        e.printStackTrace();
        }
    JFrame frame = new JFrame("Hex Layout Demo");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    HexLayoutDemo p = new HexLayoutDemo();

    frame.add(p);
    frame.pack();
    frame.setSize(800, 580);
    frame.setVisible(true);

    }


}
